"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const isomorphism_1 = require("./isomorphism");
const Option_1 = require("fp-ts/Option");
describe('payoutBatchItemCompositeKeyIsomorphism', () => {
    it('obeys the law', () => {
        const s = 'userId1:stripe:USD';
        const r = isomorphism_1.payoutBatchItemCompositeKeyIsomorphism.from(s);
        expect(isomorphism_1.payoutBatchItemCompositeKeyIsomorphism.to(r)).toEqual(s);
    });
    it('handles wrong provider', () => {
        const s = 'userId1:elonmusk:USD';
        expect(() => isomorphism_1.payoutBatchItemCompositeKeyIsomorphism.from(s)).toThrow('invalid_enum_value');
    });
    it('handles wrong currency', () => {
        const s = 'userId1:stripe:CHOCOLATE';
        expect(() => isomorphism_1.payoutBatchItemCompositeKeyIsomorphism.from(s)).toThrow('invalid_enum_value');
    });
    it('canot parse partially if not asked nicely', () => {
        const s = 'userId1:stripe:USD:::STUFF';
        expect(() => (0, isomorphism_1.payoutBatchItemCompositeKeyIsomorphismFromParseArgs)(true)(s)).toThrow('invalid serialized PayoutBatchItemCompositeKey');
    });
    // we reuse the code for other isomorphisms because it's convenient
    it('can parse partially if we ask nicely', () => {
        const s = 'userId1:stripe:USD:::STUFF';
        expect(isomorphism_1.parser.parse((0, isomorphism_1.payoutBatchItemCompositeKeyIsomorphismFromParseArgs)(false)(s))).toEqual({
            userId: 'userId1',
            originType: 'stripe',
            currency: 'USD',
        });
    });
    describe('lineable version', () => {
        it('doesnt accept non-lineable items and accepts lineable items', () => {
            const s1 = 'userId1:paypal:USD';
            const s2 = 'userId1:stripe:USD';
            expect(() => isomorphism_1.payoutBatchItemCompositeKeyLineableIsomorphism.from(s1)).toThrow('invalid_enum_value');
            isomorphism_1.payoutBatchItemCompositeKeyLineableIsomorphism.from(s2);
        });
    });
    describe('non-lineable version', () => {
        it('doesnt accept lineable items and accepts non-lineable items', () => {
            const s1 = 'userId1:paypal:USD';
            const s2 = 'userId1:stripe:USD';
            expect(() => isomorphism_1.payoutBatchItemCompositeKeyNonLineableIsomorphism.from(s2)).toThrow('invalid_enum_value');
            isomorphism_1.payoutBatchItemCompositeKeyNonLineableIsomorphism.from(s1);
        });
    });
});
describe('note isomorphisim', () => {
    it('obeys the law', () => {
        const key = 'batchKey';
        const note = 'hello';
        expect((0, isomorphism_1.extractPayoutBatchKeyFromPaypalMassPayoutLineNote)((0, isomorphism_1.makePaypalMassPayoutLineNote)(key)(note))).toEqual((0, Option_1.some)(key));
    });
    it('is idempotent', () => {
        const key = 'batchKey';
        const note = 'hello';
        const first = (0, isomorphism_1.makePaypalMassPayoutLineNote)(key)(note);
        const second = (0, isomorphism_1.makePaypalMassPayoutLineNote)(key)(first);
        expect(second).toEqual(first);
    });
    it('handles a legacy note', () => {
        const note = 'hello';
        expect((0, isomorphism_1.extractPayoutBatchKeyFromPaypalMassPayoutLineNote)(note)).toEqual(Option_1.none);
    });
    it('handles a key that isnt in the start as non key', () => {
        const note = ' {keykey} hello';
        expect((0, isomorphism_1.extractPayoutBatchKeyFromPaypalMassPayoutLineNote)(note)).toEqual(Option_1.none);
    });
    it('handles garbage tokens', () => {
        const note = '{} hello';
        expect((0, isomorphism_1.extractPayoutBatchKeyFromPaypalMassPayoutLineNote)(note)).toEqual(Option_1.none);
    });
});
