import {
  extractPayoutBatchKeyFromPaypalMassPayoutLineNote,
  makePaypalMassPayoutLineNote,
  parser,
  payoutBatchItemCompositeKeyIsomorphism,
  payoutBatchItemCompositeKeyIsomorphismFromParseArgs,
  payoutBatchItemCompositeKeyLineableIsomorphism,
  payoutBatchItemCompositeKeyNonLineableIsomorphism,
  PaypalMassPayoutLineNote,
} from './isomorphism';
import { PayoutBatchItemCompositeKeySerialized, PayoutBatchKey } from './types';
import { none, some } from 'fp-ts/Option';

describe('payoutBatchItemCompositeKeyIsomorphism', () => {
  it('obeys the law', () => {
    const s = 'userId1:stripe:USD' as PayoutBatchItemCompositeKeySerialized;
    const r = payoutBatchItemCompositeKeyIsomorphism.from(s);
    expect(payoutBatchItemCompositeKeyIsomorphism.to(r)).toEqual(s);
  });
  it('handles wrong provider', () => {
    const s = 'userId1:elonmusk:USD' as PayoutBatchItemCompositeKeySerialized;
    expect(() => payoutBatchItemCompositeKeyIsomorphism.from(s)).toThrow('invalid_enum_value');
  });
  it('handles wrong currency', () => {
    const s = 'userId1:stripe:CHOCOLATE' as PayoutBatchItemCompositeKeySerialized;
    expect(() => payoutBatchItemCompositeKeyIsomorphism.from(s)).toThrow('invalid_enum_value');
  });
  it('canot parse partially if not asked nicely', () => {
    const s = 'userId1:stripe:USD:::STUFF' as `${PayoutBatchItemCompositeKeySerialized}${string}`;
    expect(() => payoutBatchItemCompositeKeyIsomorphismFromParseArgs(true)(s)).toThrow(
      'invalid serialized PayoutBatchItemCompositeKey'
    );
  });
  // we reuse the code for other isomorphisms because it's convenient
  it('can parse partially if we ask nicely', () => {
    const s = 'userId1:stripe:USD:::STUFF' as `${PayoutBatchItemCompositeKeySerialized}${string}`;
    expect(parser.parse(payoutBatchItemCompositeKeyIsomorphismFromParseArgs(false)(s))).toEqual({
      userId: 'userId1',
      originType: 'stripe',
      currency: 'USD',
    });
  });
  describe('lineable version', () => {
    it('doesnt accept non-lineable items and accepts lineable items', () => {
      const s1 = 'userId1:paypal:USD' as PayoutBatchItemCompositeKeySerialized<'stripe'>;
      const s2 = 'userId1:stripe:USD' as PayoutBatchItemCompositeKeySerialized<'stripe'>;
      expect(() => payoutBatchItemCompositeKeyLineableIsomorphism.from(s1)).toThrow('invalid_enum_value');
      payoutBatchItemCompositeKeyLineableIsomorphism.from(s2);
    });
  });
  describe('non-lineable version', () => {
    it('doesnt accept lineable items and accepts non-lineable items', () => {
      const s1 = 'userId1:paypal:USD' as PayoutBatchItemCompositeKeySerialized<'paypal'>;
      const s2 = 'userId1:stripe:USD' as PayoutBatchItemCompositeKeySerialized<'paypal'>;
      expect(() => payoutBatchItemCompositeKeyNonLineableIsomorphism.from(s2)).toThrow('invalid_enum_value');
      payoutBatchItemCompositeKeyNonLineableIsomorphism.from(s1);
    });
  });
});

describe('note isomorphisim', () => {
  it('obeys the law', () => {
    const key = 'batchKey' as PayoutBatchKey;
    const note = 'hello';
    expect(extractPayoutBatchKeyFromPaypalMassPayoutLineNote(makePaypalMassPayoutLineNote(key)(note))).toEqual(
      some(key)
    );
  });
  it('is idempotent', () => {
    const key = 'batchKey' as PayoutBatchKey;
    const note = 'hello';
    const first = makePaypalMassPayoutLineNote(key)(note);
    const second = makePaypalMassPayoutLineNote(key)(first);
    expect(second).toEqual(first);
  });
  it('handles a legacy note', () => {
    const note = 'hello';
    expect(extractPayoutBatchKeyFromPaypalMassPayoutLineNote(note as PaypalMassPayoutLineNote)).toEqual(none);
  });
  it('handles a key that isnt in the start as non key', () => {
    const note = ' {keykey} hello';
    expect(extractPayoutBatchKeyFromPaypalMassPayoutLineNote(note as PaypalMassPayoutLineNote)).toEqual(none);
  });
  it('handles garbage tokens', () => {
    const note = '{} hello';
    expect(extractPayoutBatchKeyFromPaypalMassPayoutLineNote(note as PaypalMassPayoutLineNote)).toEqual(none);
  });
});
